var CommonFunc = {

  ICON_MODE: "potosidebar.icon_mode",
	LARGE_ICONS: "potosidebar.large_icons",
	OPEN_METHOD: "potosidebar.open_method",
	TOUCH_DELAY: "potosidebar.touch_delay",

// ++++++++++ ++++++++++ preferences ++++++++++ ++++++++++

	setPrefValue : function(aPrefString, aPrefType, aValue) {
		var nsISupportsString = Components.interfaces.nsISupportsString;
		var xpPref = Components.classes["@mozilla.org/preferences;1"].getService(Components.interfaces.nsIPrefBranch);

		var prefType = xpPref.getPrefType(aPrefString);

		try {
			switch(aPrefType) {
				case "wstr":
					var string = Components.classes['@mozilla.org/supports-string;1'].createInstance(nsISupportsString);
					string.data = aValue;
					return xpPref.setComplexValue(aPrefString, nsISupportsString, string);
					break;
				case "str":
					return xpPref.setCharPref(aPrefString, aValue);
					break;
				case "int":
					aValue = parseInt(aValue);
					return xpPref.setIntPref(aPrefString, aValue);
					break;
				case "bool":
				default:
					if(typeof(aValue) == "string") {
						aValue = (aValue == "true");
					}
					return xpPref.setBoolPref(aPrefString, aValue);
					break;
			}
		} catch(e) {
		}
		return null;
	},

	getPrefValue : function(aPrefString, aPrefType, aDefault) {
		var nsISupportsString = Components.interfaces.nsISupportsString;
		var xpPref = Components.classes["@mozilla.org/preferences;1"].getService(Components.interfaces.nsIPrefBranch);

		if(xpPref.getPrefType(aPrefString) == xpPref.PREF_INVALID) {
			return aDefault;
		}
		try {
			switch (aPrefType) {
				case "wstr":
					return xpPref.getComplexValue(aPrefString, nsISupportsString).data;
					break;
				case "str":
					return xpPref.getCharPref(aPrefString).toString();
					break;
				case "int":
					return xpPref.getIntPref(aPrefString);
					break;
				case "bool":
				default:
					return xpPref.getBoolPref(aPrefString);
					break;
			}
		} catch(e) {
		}
		return aDefault;
	},

	clearPref: function(aPrefString) {
		var xpPref = Components.classes["@mozilla.org/preferences;1"].getService(Components.interfaces.nsIPrefBranch);

		try {
			xpPref.clearUserPref(aPrefString);
			return true;
		} catch(e) {
			return false;
		}
	},

		// remove all preferences
	removePrefs: function() {
		var xpPref = Components.classes["@mozilla.org/preferences;1"].getService(Components.interfaces.nsIPref);

		var prefBranch = xpPref.getBranch("sage.");

		try {
			prefBranch.deleteBranch("");
			return true;
		} catch(e) {
			return false;
		}
	},

	addPrefListener: function(aPrefString, aFunc) {
		var prefObserver;
		try {
			prefObserver = {
				domain: aPrefString,
				observe: aFunc
			};

			var pbi = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranchInternal);
			pbi.addObserver(prefObserver.domain, prefObserver, false);
		} catch(e) {
			alert(e);
			prefObserver = null;
		}

		return prefObserver;
	},

	removePrefListener: function(aObserver) {
		var prefObserver;
		try {
			var pbi = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranchInternal);
			pbi.removeObserver(aObserver.domain, aObserver);
		} catch(e) {
			alert(e)
		}
	},

	// takes a version string, returns an integer triple containing (major version, minor version, patch level)
	versionStrDecode: function(versionStr) {
		var regexp = /([0-9]*)\.([0-9]*)\.([0-9]*)/;
		var result = regexp.exec(versionStr);
		return Array(parseInt(result[1]), parseInt(result[2]), parseInt(result[3]));
	},

	// takes a version triple, returns an integer
	versionToInt: function(versionTriple) {
		return versionTriple[0]*100 + versionTriple[1]*10 + versionTriple[2];
	},

	// takes two version triples, returns 1 if the first is more recent, 0 otherwise
	versionCompare: function(versionA, versionB) {
		if(this.versionToInt(versionA) > this.versionToInt(versionB)) {
			return 1;
		} else {
			return 0;
		}
	},

	// takes a version triple, returns a formatted version string
	versionString: function(version, pretty) {
		var formatted;
		if(pretty) {
			formatted = version[0].toString() + '.' + version[1].toString();
			formatted += version[2] != 0 ? "." + version[2] : ""
		} else {
			formatted = version[0].toString() + '.' + version[1].toString() + '.' + version[2].toString();
		}
		return formatted;
	}
}
